import pandas as pd
from ..utils import *
from ..path import *

GroundTruth = 'GroundTruth'
Predict = 'Predict'
DEFAULT_OUTPUT_FILE_NAME = 'prf'


def calculate_prf(df, ground_truth_colunms, predict_columns, breakdown_column=None, weight_column=None, drop_na=True, save_to_file=True, output_file_name=None):
	"""
	Calculate the precision, recall, F1 of the prediction
	:param df: input dataframe
	:param ground_truth_colunms: ground truth columns
	:param predict_columns: predict columns
	:param breakdown_column: breakdown column
	:param weight_column: weight for the calculation
	:param drop_na: drop the rows with missing values in the ground_truth_colunms and predict_columns
	:param save_to_file: save the result to a csv file
	:param output_file_name: the name of the file to save the result
	:return: a dataframe with precision, recall, F1 of the prediction
	"""
	merged_prf = []
	# check the type of ground_truth_colunms and predict_columns if they are not list, convert them to list
	if not isinstance(ground_truth_colunms, list):
		ground_truth_colunms = [ground_truth_colunms]
	if not isinstance(predict_columns, list):
		predict_columns = [predict_columns]

	# check the length of ground_truth_colunms and predict_columns are the same
	assert len(ground_truth_colunms) == len(predict_columns), "The length of ground_truth_colunms and predict_columns should be the same"
	for i in range(len(ground_truth_colunms)):
		prf_df = calculate_single_pair_prf(df, ground_truth_colunms[i], predict_columns[i], breakdown_column=breakdown_column, weight_column=weight_column, drop_na=drop_na)
		merged_prf.append(prf_df)
	merged_prf = pd.concat(merged_prf)
	save_df_to_csv(merged_prf, save_to_file, default_output_file_name=DEFAULT_OUTPUT_FILE_NAME, custom_output_file_name=output_file_name)
	return merged_prf

def calculate_single_pair_prf(df, ground_truth_colunm, predict_column, breakdown_column=None, weight_column=None, drop_na=True):
	"""
	Calculate the precision, recall, F1 of the prediction
	:param df: input dataframe
	:param ground_truth_colunm: ground truth column
	:param predict_column: predict column
	:param breakdown_column: breakdown column
	:param weight_column: weight for the calculation
	:param drop_na: drop the rows with missing values in the ground_truth_colunm and predict_column
	:return: a dataframe with precision, recall, F1 of the prediction
	"""
	# # check df is a pandas dataframe, ground_truth_colunm and predict_column, breakdown_column are in the df
	# assert isinstance(df, pd.DataFrame), "df should be a pandas dataframe"
	# assert ground_truth_colunm in df.columns, f"{ground_truth_colunm} should be in the dataframe"
	# assert predict_column in df.columns, f"{predict_column} should be in the dataframe"
	# if breakdown_column is not None:
	# 	assert breakdown_column in df.columns, f"{breakdown_column} should be in the dataframe"
	
	# remove rows with missing values in the two columns
	if drop_na:
		df = drop_na_raw(df, [ground_truth_colunm, predict_column])

	# check the value of ground_truth_colunm and predict_column are in [0, 1]
	assert df[ground_truth_colunm].isin([0, 1]).all(), f"{ground_truth_colunm} should only contain 0 or 1"
	assert df[predict_column].isin([0, 1]).all(), f"{predict_column} should only contain 0 or 1"
	
	if breakdown_column is None:
		prf_df = calculate_prf_single(df, ground_truth_colunm, predict_column, weight_column)
		prf_df = prf_df.to_frame().T
	else:
		prf_df = df.groupby(breakdown_column).apply(lambda x: calculate_prf_single(x, ground_truth_colunm, predict_column, weight_column)).reset_index()
		# put the GroundTruth and Predict columns to the front
		cols = prf_df.columns.tolist()
		cols.remove(GroundTruth)
		cols.remove(Predict)
		prf_df = prf_df[[GroundTruth, Predict] + cols]
	return prf_df


def calculate_prf_single(df, ground_truth_colunm, predict_column, weight_column):
	"""
	Calculate the precision, recall, F1 of the prediction for a single group
	:param df: input dataframe
	:param ground_truth_colunm: ground truth column
	:param predict_column: predict column
	:param weight_column: weight for the calculation
	"""
	ground_truth = df[ground_truth_colunm].values.tolist()
	predict = df[predict_column].values.tolist()
	if weight_column is not None:
		weights = df[weight_column].values.tolist()
	TP = 0
	FP = 0
	TN = 0
	FN = 0
	for i in range(len(ground_truth)):
		weight = 1
		if weight_column is not None:
			weight = weights[i]
		if ground_truth[i] == 1 and predict[i] == 1:
			TP += weight
		elif ground_truth[i] == 0 and predict[i] == 1:
			FP += weight
		elif ground_truth[i] == 1 and predict[i] == 0:
			FN += weight
		elif ground_truth[i] == 0 and predict[i] == 0:
			TN += weight
		else:
			print("ground_truth and predict should be 0 or 1")
			raise ValueError(f"ground_truth: {ground_truth[i]}, predict: {predict[i]}")
	if TP + FP == 0:
		precision = 0
	else:
		precision = TP / (TP + FP)
	if TP + FN == 0:
		recall = 0
	else:
		recall = TP / (TP + FN)
	if precision + recall == 0:
		f1 = 0
	else:
		f1 = 2 * precision * recall / (precision + recall)
	ground_truth_positive = TP + FN
	ground_truth_positive_ratio = ground_truth_positive / len(ground_truth)
	predict_positive = TP + FP
	predict_positive_ratio = predict_positive / len(predict)
	ACC = (TP + TN) / (TP + FP + TN + FN)
	FPR = FP / (FP + TN)
	return pd.Series({GroundTruth: ground_truth_colunm, Predict: predict_column, 'TP': TP, 'FP': FP, 'TN': TN, 'FN': FN, 'Precision': precision, 'Recall': recall, 'F1': f1, 'ACC': ACC, 'FPR': FPR,
				   'ground_truth_positive': ground_truth_positive, 'ground_truth_positive_ratio': ground_truth_positive_ratio, 
				   'predict_positive': predict_positive, 'predict_positive_ratio': predict_positive_ratio
				   })





import time
import pandas as pd
from ..path import *

def save_df_to_csv(df, save_to_file, default_output_file_name, custom_output_file_name=None):
	"""
	Save the dataframe to a csv file
	:param df: DataFrame
	:param save_to_file: if save path
	:param default_output_file_name: default output file name
	:param custom_output_file_name: custom output file name
	"""
	if not save_to_file:
		return
	
	path = get_output_dir()
	if custom_output_file_name is not None:
		path += f'/{custom_output_file_name}.csv'
	else:
		default_output_file_name += f'_{time.strftime("%Y%m%d%H%M%S")}'
		path += f'/{default_output_file_name}.csv'

	df.to_csv(path, index=False)
	print(f">>>> save dataframe result to {path}")

def drop_na_raw(df, columns):
	"""
	Drop the rows with na value in the columns
	:param df: DataFrame
	:param columns: list of column names
	:return: DataFrame
	"""
	all_rows = df.shape[0]
	df = df.dropna(subset=columns)
	drop_rows = all_rows - df.shape[0]
	print(f'>>>> all rows = {all_rows}, drop rows = {drop_rows} in {columns}')
	return df



import os
import pandas as pd
import numpy as np
from ..visualization import *
from ..path import *
from ..utils import *
from ..metrics import *

COUNT = 'Count'
RATIO = 'Ratio'
SHIFT = 'Shift'
GROUP_TOTAL = 'group_total'
GROUP_RATIO = 'group_ratio'
TOTAL_RATIO = 'total_ratio'
COLUMN1_COLUMN2 = 'column1_column2'

def single_feature_analysis(df, column, drop_na=True, save_to_file=True, output_file_name=None):
	"""
	Analyze the single feature
	:param df: DataFrame
	:param column: column name
	:param drop_na: drop the rows with no value
	:param save_to_file: save the result to files
	:param output_file_name: the name of the file to save the result
	:return: analysis
	"""
	default_output_file_name = f'{column}_distribution'

	if drop_na:
		df = drop_na_raw(df, [column])

	feature_analysis = df.groupby(column)[column].value_counts().reset_index(name=COUNT)
	total_count = feature_analysis[COUNT].sum()
	feature_analysis[RATIO] = feature_analysis[COUNT] / total_count

	bar_plot(data=feature_analysis, xlabel=column, ylabel=COUNT, title=f'Distribution of {column}', save_to_file=save_to_file, default_output_file_name=default_output_file_name, custom_output_file_name=output_file_name)
	pie_plot(data=feature_analysis, labels=feature_analysis[column], sizes=feature_analysis[COUNT], title=f'Distribution of {column}', save_to_file=save_to_file, default_output_file_name=default_output_file_name, custom_output_file_name=output_file_name)
	save_df_to_csv(feature_analysis, save_to_file, default_output_file_name=default_output_file_name, custom_output_file_name=output_file_name)
	return feature_analysis


def pivot_table(df, column1, column2, drop_na=True, save_to_file=True, output_file_name=None):
	"""
	Create a pivot table for two columns
	:param df: DataFrame
	:param column1: column name
	:param column2: column name
	:param drop_na: drop the rows with no value
	:param save_to_file: save the result to files
	:param output_file_name: the name of the file to save the result
	:return: analysis
	"""
	default_output_file_name = f'{column1}_{column2}_pivot_table'
	count_default_output_file_name = f'{column1}_{column2}_pivot_table_count'
	group_ratio_default_output_file_name = f'{column1}_{column2}_pivot_table_group_ratio'
	count_custom_output_file_name = None if not output_file_name else output_file_name + "_pivot_table_count"
	group_ratio_custom_output_file_name = None if not output_file_name else output_file_name + "_pivot_table_group_ratio"
	
	if drop_na:
		df = drop_na_raw(df, [column1, column2])

	pivot_analysis = df.groupby([column1, column2]).size().reset_index(name=COUNT)
	pivot_analysis = pivot_analysis.sort_values(by=COUNT, ascending=False)
	total_count = pivot_analysis[COUNT].sum()
	pivot_analysis[TOTAL_RATIO] = pivot_analysis[COUNT] / total_count

	column1_df = pivot_analysis.groupby([column1])[COUNT].sum().reset_index()
	column1_df = column1_df.rename(columns={COUNT: GROUP_TOTAL})
	pivot_analysis = pd.merge(pivot_analysis, column1_df, on=column1, how='left')
	pivot_analysis[GROUP_RATIO] = pivot_analysis[COUNT] / pivot_analysis[GROUP_TOTAL]

	group_pivot_analysis = pivot_analysis.copy()
	group_pivot_analysis[COLUMN1_COLUMN2] = group_pivot_analysis[column1].astype(str) + "&" + group_pivot_analysis[column2].astype(str)

	pivot_bar_plot(data=pivot_analysis, index=column1, columns=column2, values=COUNT, title=f'Count of {column1} and {column2}', save_to_file=save_to_file, default_output_file_name=count_default_output_file_name, custom_output_file_name=count_custom_output_file_name)
	pivot_bar_plot(data=pivot_analysis, index=column1, columns=column2, values=GROUP_RATIO, title=f'Group Ratio  {column2}', save_to_file=save_to_file, default_output_file_name=group_ratio_default_output_file_name, custom_output_file_name=group_ratio_custom_output_file_name)
	pie_plot(data=group_pivot_analysis, labels=group_pivot_analysis[COLUMN1_COLUMN2], sizes=group_pivot_analysis[COUNT], title=f'Total Ratio of {column1} + {column2}', save_to_file=save_to_file, default_output_file_name=default_output_file_name, custom_output_file_name=output_file_name)
	save_df_to_csv(pivot_analysis, save_to_file, default_output_file_name=default_output_file_name, custom_output_file_name=output_file_name)
	return pivot_analysis


def feature_shift_analysis(df, column1, column2, gen_pivot_table=True, drop_na=True, save_to_file=True, output_file_name=None):
	"""
	Analyze the shift of two columns
	:param df: DataFrame
	:param column1: column name
	:param column2: column name
	:param gen_pivot_table: generate pivot table
	:param drop_na: drop the rows with no value
	:param save_to_file: save the result to files
	:param output_file_name: the name of the file to save the result
	:return: analysis
	"""
	default_output_file_name = f'{column1}_{column2}_shift'
	if gen_pivot_table:
		pivot_table(df, column1, column2, drop_na, save_to_file, output_file_name)

	if drop_na:
		df = drop_na_raw(df, [column1, column2])

	df[SHIFT] = df[column1] != df[column2]
	overall_shift = df[SHIFT].value_counts().reset_index(name=COUNT)
	overall_shift[RATIO] = overall_shift[COUNT] / overall_shift[COUNT].sum()

	pie_plot(data=overall_shift, labels=overall_shift[SHIFT], sizes=overall_shift[COUNT], title=f'Shift of {column1} and {column2}', save_to_file=save_to_file, default_output_file_name=default_output_file_name, custom_output_file_name=output_file_name)
	save_df_to_csv(overall_shift, save_to_file, default_output_file_name=default_output_file_name, custom_output_file_name=output_file_name)
	return overall_shift


def feature_correlation_coefficient(df, column, drop_na=True, save_to_file=True, output_file_name=None):
	"""
	Calculate the correlation coefficient of the column with other columns
	:param df: DataFrame
	:param column: column name
	:param drop_na: drop the rows with no value
	:param save_to_file: save the result to files
	:param output_file_name: the name of the file to save the result
	:return: correlation coefficient
	"""
	default_output_file_name = f'{column}_correlation_coefficient'
	if drop_na:
		df = drop_na_raw(df, [column])
	
	# filter the columns with numeric values
	df = df.select_dtypes(include=[np.number])

	correlation_coefficient = df.corr()[column].reset_index()
	correlation_coefficient = correlation_coefficient.rename(columns={column: 'correlation_coefficient'})
	correlation_coefficient = correlation_coefficient.sort_values(by='correlation_coefficient', ascending=False)
	save_df_to_csv(correlation_coefficient, save_to_file, default_output_file_name=default_output_file_name, custom_output_file_name=output_file_name)
	return correlation_coefficient


def threshold_selection(df, ground_truth_column, predict_column, need_sigmoid=True, threshold_list=None,  line_chart_metrics=None, save_to_file=True, output_file_name=None):
	"""
	Analyze the threshold of the prediction
	:param df: DataFrame
	:param ground_truth_column: ground truth column
	:param predict_column: predict column
	:param need_sigmoid: whether need sigmoid
	:param threshold_list: threshold list, the default value is np.linspace(0, 1, 21)
	:param line_chart_metrics: line chart metrics, the default value is ['recall', 'precision', 'f1', 'FPR']
	:param save_to_file: save the result to files
	:param output_file_name: the name of the file to save the result
	"""
	default_output_file_name = f'{ground_truth_column}_{predict_column}_threshold_selection'
	optimum_thresholds_default_output_file_name = f'{default_output_file_name}_optimum_thresholds'
	optimum_thresholds_output_file_name = None if not output_file_name else output_file_name + "_optimum_thresholds" 

	# check the value of ground_truth_column are in [0, 1], predict_column are float
	assert set(df[ground_truth_column].unique()) == {0, 1}, f"The value of {ground_truth_column} should be in [0, 1]"
	assert df[predict_column].dtype == float, f"The value of {predict_column} should be float"
	if need_sigmoid:
		df[predict_column] = df[predict_column].apply(lambda x: np.exp(x) / (1 + np.exp(x)))
		print(f">>>> IMPORTANT!!!! The sigmoid function is applied to {predict_column}")

	df = drop_na_raw(df, [ground_truth_column, predict_column])

	if threshold_list is None:
		threshold_list = np.linspace(0, 1, 21)
		print(f">>>> The default threshold list is {threshold_list}")
	threshold_list = [round(threshold, 2) for threshold in threshold_list]

	df = df[[ground_truth_column, predict_column]]
	merge_df = []
	for threshold in threshold_list:
		tmp_df = df.copy()
		tmp_df['threshold'] = threshold
		tmp_df[predict_column] = df[predict_column] > threshold
		merge_df.append(tmp_df)
	merge_df = pd.concat(merge_df)
	prf_df = calculate_prf(merge_df, ground_truth_column, predict_column, breakdown_column='threshold', save_to_file=save_to_file, output_file_name=output_file_name)
	
	print(f">>>> line chart metrics should be in {prf_df.columns}")
	if line_chart_metrics is None:
		line_chart_metrics = ['recall', 'precision', 'f1', 'FPR']
		print(f">>>> The default line chart metrics are {line_chart_metrics}")

	line_chart(prf_df, 'threshold', line_chart_metrics, title=f'{",".join(line_chart_metrics)} vs {predict_column} Threshold', x_label="threshold", y_label="value",  save_to_file=save_to_file, default_output_file_name=default_output_file_name, custom_output_file_name=output_file_name)
	
	optimum_thresholds = {"Metrics": [], "Metrics_Value": [], "Threshold": []}
	for metric in line_chart_metrics:
		optimum_thresholds["Metrics"].append(metric)
		optimum_thresholds["Metrics_Value"].append(prf_df[metric].max())
		optimum_thresholds["Threshold"].append(prf_df[prf_df[metric] == prf_df[metric].max()]['threshold'].values[0])
	optimum_thresholds = pd.DataFrame(optimum_thresholds)

	save_df_to_csv(optimum_thresholds, save_to_file, default_output_file_name=optimum_thresholds_default_output_file_name, custom_output_file_name=optimum_thresholds_output_file_name)
	return prf_df







	



import time
import numpy as np
import seaborn as sns
import matplotlib.pyplot as plt

from ..path import *

def bar_plot(data, xlabel=None, ylabel=None, title=None, save_to_file=True, default_output_file_name=None, custom_output_file_name=None):
	"""Plot a bar plot."""
	if not save_to_file:
		return
	plt.clf()
	path = get_output_dir()
	if custom_output_file_name:
		path += f'/{custom_output_file_name}_bar_chart.png' 
	else:
		path += f'/{default_output_file_name}_bar_chart_{time.strftime("%Y%m%d%H%M%S")}.png'

	ax = sns.barplot(x=xlabel, y=ylabel, data=data)
	for p in ax.patches:
		p.set_height(p.get_height())
		ax.text(p.get_x() + p.get_width() / 2., p.get_height(), '%d' % int(p.get_height()), ha='center', va='bottom')
	plt.title(title)
	plt.savefig(path)
	print(f">>>> Plot saved at {path}")


def pie_plot(data, labels=None, sizes=None, title=None, save_to_file=True, default_output_file_name=None, custom_output_file_name=None):
	"""Plot a pie plot."""
	if not save_to_file:
		return
	plt.clf()
	path = get_output_dir()
	if custom_output_file_name:
		path += f'/{custom_output_file_name}_pie_chart.png'
	else:
		path += f'/{default_output_file_name}_pie_chart_{time.strftime("%Y%m%d%H%M%S")}.png'

	fig1, ax1 = plt.subplots()
	ax1.pie(sizes, labels=labels, autopct='%1.1f%%', shadow=True, startangle=90)
	ax1.axis('equal')
	plt.title(title)
	plt.savefig(path)
	print(f">>>> Plot saved at {path}")


def line_chart(data, x=None, y=None, title=None, labels=None, x_label=None, y_label=None, save_to_file=True, default_output_file_name=None, custom_output_file_name=None, if_clf=True):
	"""Plot a line chart."""
	if not save_to_file:
		return
	if if_clf:
		plt.clf()
	path = get_output_dir()
	if custom_output_file_name:
		path += f'/{custom_output_file_name}_line_chart.png'
	else:
		path += f'/{default_output_file_name}_line_chart_{time.strftime("%Y%m%d%H%M%S")}.png'

	# draw line chart for recall and precision
	if not isinstance(x, list):
		x = [x]
	if not isinstance(y, list):
		y = [y]
	if len(x) < len(y):
		if len(x) == 1:
			x = x * len(y)
	if labels is None:
		labels = y
	for x_, y_, label in zip(x, y, labels):
		plt.plot(data[x_], data[y_], label=label)
	plt.xlabel(x_label)
	plt.ylabel(y_label)
	plt.title(title)
	plt.legend()
	plt.savefig(path)
	print(f">>>> Plot saved at {path}")


def pivot_bar_plot(data, index=None, columns=None, values=None, title=None, save_to_file=True, default_output_file_name=None, custom_output_file_name=None):
	"""Plot a bar plot."""
	if not save_to_file:
		return
	plt.clf()
	path = get_output_dir()
	if custom_output_file_name:
		path += f'/{custom_output_file_name}_pivot_bar_chart.png'
	else:
		path += f'/{default_output_file_name}_pivot_bar_chart_{time.strftime("%Y%m%d%H%M%S")}.png'

	data = data.pivot(index=index, columns=columns, values=values)
	ax = data.plot(kind='bar', stacked=False)
	for p in ax.patches:
		p.set_height(p.get_height())
		if 'ratio' in values.lower():
			ax.text(p.get_x() + p.get_width() / 2., p.get_height(), '%d%%' % int(p.get_height() * 100), ha='center', va='bottom')
		else:
			ax.text(p.get_x() + p.get_width() / 2., p.get_height(), '%d' % int(p.get_height()), ha='center', va='bottom')
	plt.title(title)
	plt.savefig(path)
	print(f">>>> Plot saved at {path}")



if __name__ == '__main__':
	import pandas as pd
	# data = {'Market': ['en_us', 'en_us', 'zh_cn'], 'Prediction': [0, 1, 0], 'Count': [1, 1, 2]}
	# df = pd.DataFrame(data)
	# # single_bar_plot(df, 'Prediction', 'Count', 'Distribution of Prediction', 'Prediction_distribution.png')
	# pivot_bar_plot(df, 'Market', 'Prediction', 'Distribution of Prediction', 'Prediction_distribution.png')


	data = {'threshold': [0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1], 
		 'recall': [1, 0.9, 0.8, 0.7, 0.6, 0.5, 0.4, 0.3, 0.2, 0.1, 0], 
		 'precision': [0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1]}
	df = pd.DataFrame(data)

	# draw line chart for recall and precision
	line_chart(df, 'threshold', ['recall', 'precision'], 'Recall and Precision', 'Recall_Precision.png')


import pandas as pd
from sklearn.metrics import roc_auc_score, roc_curve
from ..utils import *
from ..path import *
from ..visualization import line_chart

GroundTruth = 'GroundTruth'
Predict = 'Predict'
DEFAULT_OUTPUT_FILE_NAME = 'ranking'


def calculate_auc(df, ground_truth_colunms, predict_columns, save_to_file=True, drop_na=True, output_file_name=None):
	"""
	Calculate the auc and draw the roc curve
	:param df: input dataframe
	:param ground_truth_colunms: ground truth columns
	:param predict_columns: predict columns
	:param drop_na: drop the rows with missing values in the ground_truth_colunms and predict_columns
	:param save_to_file: save the result to a csv file
	:param output_file_name: the name of the file to save the result
	:return: a dataframe with precision, recall, F1 of the prediction
	"""
	merged_auc = {}
	merged_roc_df = []
	fpr_columns = []
	tpr_columns = []
	# check the type of ground_truth_colunms and predict_columns if they are not list, convert them to list
	if not isinstance(ground_truth_colunms, list):
		ground_truth_colunms = [ground_truth_colunms]
	if not isinstance(predict_columns, list):
		predict_columns = [predict_columns]

	# check the length of ground_truth_colunms and predict_columns are the same
	assert len(ground_truth_colunms) == len(predict_columns), "The length of ground_truth_colunms and predict_columns should be the same"
	for i in range(len(ground_truth_colunms)):
		auc, roc_df = calculate_auc_single(df, ground_truth_colunms[i], predict_columns[i], drop_na=drop_na)
		merged_auc[f'{ground_truth_colunms[i]}_{predict_columns[i]}'] = auc
		merged_roc_df.append(roc_df)
		fpr_columns.append(f'{predict_columns[i]}_fpr')
		tpr_columns.append(f'{predict_columns[i]}_tpr')
	roc_df = merged_roc_df[0]
	for i in range(1, len(merged_roc_df)):
		roc_df = pd.merge(roc_df, merged_roc_df[i], on='thresholds', how='outer')
	roc_df = roc_df.reset_index(drop=True)
	# put thresholds column to the first column
	roc_df = roc_df[['thresholds'] + fpr_columns + tpr_columns]
	save_df_to_csv(roc_df, save_to_file, default_output_file_name=DEFAULT_OUTPUT_FILE_NAME, custom_output_file_name=output_file_name)
	merged_auc = pd.DataFrame(merged_auc, index=[0])
	save_df_to_csv(merged_auc, save_to_file, default_output_file_name=DEFAULT_OUTPUT_FILE_NAME, custom_output_file_name=output_file_name)
	line_chart(roc_df, fpr_columns, tpr_columns, title="ROC", labels=predict_columns, x_label="False positive ratio", y_label="True positive ratio", save_to_file=save_to_file, default_output_file_name=DEFAULT_OUTPUT_FILE_NAME, custom_output_file_name=output_file_name)
	return merged_auc, roc_df


def calculate_auc_single(df, ground_truth_colunm, predict_column, drop_na):
	"""
	Calculate the precision, recall, F1 of the prediction for a single group
	:param df: input dataframe
	:param ground_truth_colunm: ground truth column
	:param predict_column: predict column
	:param weight_column: weight for the calculation
	"""
	if drop_na:
		df = drop_na_raw(df, [ground_truth_colunm, predict_column])

	ground_truth = df[ground_truth_colunm].values.tolist()
	predict = df[predict_column].values.tolist()
	
	# calculate the auc
	auc = roc_auc_score(ground_truth, predict)
	fpr, tpr, thresholds = roc_curve(ground_truth, predict)
	roc_df = pd.DataFrame({f'{predict_column}_fpr': fpr, f'{predict_column}_tpr': tpr, 'thresholds': thresholds})
	return auc, roc_df





import os
import pandas as pd
from ..visualization import *
from ..path import *
from ..utils import *

DEFAULT_OUTPUT_FILE_NAME = 'sample'

def custom_sample(df, sample_filter, sample_count=None, breakdown_column=None, sample_columns=None, random_state=None, drop_na=True, save_to_file=True, output_file_name=None):
	"""
	Custom sample the dataframe
	:param df: input dataframe
	:param sample_filter: sample filter
	:param sample_count: sample size
	:param breakdown_column: breakdown column
	:param sample_columns: sample columns
	:param random_state: random state
	:param drop_na: drop the rows with missing values in the sample_columns
	:param save_to_file: save the result to a csv file
	:param output_file_name: the name of the file to save the result
	:return: a dataframe with sampled data
	"""
	print(f">>>> sample_filter : {sample_filter}")
	
	if sample_columns is None or len(sample_columns) == 0:
		sample_columns = df.columns.tolist()
	if sample_count is None:
		sample_count = 20
		print(f">>>> Sample size is not specified, use default value {sample_count}")
	
	# dedup according to sample_columns
	print(f">>>> Original dataframe size: {df.shape}")
	df = df.drop_duplicates(subset=sample_columns)
	print(f">>>> Deduped dataframe size: {df.shape}")

	if drop_na:
		df = drop_na_raw(df, sample_columns)
		print(f">>>> Drop na dataframe size: {df.shape}")

	# sample based on breakdown_column 
	if breakdown_column is not None and len(breakdown_column) > 0:
		sample_df = df[sample_columns].groupby(breakdown_column).apply(lambda x: x.query(sample_filter)).reset_index(drop=True)
		sample_df = sample_df.groupby(breakdown_column).apply(lambda x: simple_sample(x, sample_count, random_state=random_state)).reset_index(drop=True)
		print(f">>>> Sampled dataframe size for each group: {sample_df.groupby(breakdown_column).size()}")
	else:
		sample_df = df[sample_columns].query(sample_filter)
		sample_df = simple_sample(sample_df, sample_count, random_state=random_state)
	print(f">>>> Sampled dataframe size: {sample_df.shape}")
	save_df_to_csv(sample_df, save_to_file, default_output_file_name=DEFAULT_OUTPUT_FILE_NAME, custom_output_file_name=output_file_name)
	return sample_df


def simple_sample(df, sample_count, random_state=None):
	"""
	Sample the dataframe
	:param df: input dataframe
	:param sample_count: sample size
	:param random_state: random state
	return: sample dataframe
	"""
	if sample_count > df.shape[0]:
		print(f">>>> Sample size is larger than the dataframe size, use the dataframe size {df.shape[0]}")
		sample_count = df.shape[0]
	df = df.sample(sample_count, random_state=random_state)
	return df

